/**
 * Copyright (c) 2011-2017 libbitcoin developers (see AUTHORS)
 *
 * This file is part of libbitcoin.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <bitcoin/explorer/commands/ek-to-address.hpp>

#include <cstdint>
#include <iostream>
#include <bitcoin/bitcoin.hpp>
#include <bitcoin/explorer/define.hpp>

namespace libbitcoin {
namespace explorer {
namespace commands {
using namespace bc::wallet;

console_result ek_to_address::invoke(std::ostream& output, std::ostream& error)
{
#ifdef WITH_ICU
    const auto& passphrase = get_passphrase_argument();
    const auto& key = get_ek_private_key_argument();

    bool compressed;
    uint8_t version;
    ec_secret secret;
    if (!decrypt(secret, version, compressed, key, passphrase))
    {
        error << BX_EK_TO_ADDRESS_INVALID_PASSPHRASE << std::endl;
        return console_result::failure;
    }

    ec_compressed point;
    secret_to_public(point, secret);
    const payment_address address({ point, compressed }, version);

    output << address << std::endl;
    return console_result::okay;
#else
    error << BX_EK_TO_ADDRESS_REQUIRES_ICU << std::endl;
    return console_result::failure;
#endif
}

} //namespace commands
} //namespace explorer
} //namespace libbitcoin
